import numpy as np
from skimage.io import imsave
from tifffile import imwrite

from base.process import make_rgb


# TODO: move this as method in base.accessors
def write_patch_to_file(where, fname, yxcz):
    ext = fname.split('.')[-1].upper()
    where.mkdir(parents=True, exist_ok=True)

    if ext == 'PNG':
        assert yxcz.dtype == 'uint8', f'Invalid data type {yxcz.dtype}'
        assert yxcz.shape[2] <= 3, f'Cannot export images with more than 3 channels as PNGs'
        assert yxcz.shape[3] == 1, f'Cannot export z-stacks as PNGs'
        if yxcz.shape[2] == 1:
            outdata = yxcz[:, :, 0, 0]
        elif yxcz.shape[2] == 2:  # add a blank blue channel
            outdata = make_rgb(yxcz)
        else: # preserve RGB order
            outdata = yxcz[:, :, :, 0]
        imsave(where / fname, outdata, check_contrast=False)
        return True

    elif ext in ['TIF', 'TIFF']:
        zcyx = np.moveaxis(yxcz, [3, 2, 0, 1], [0, 1, 2, 3])
        imwrite(where / fname, zcyx, imagej=True)
        return True

    else:
        raise Exception(f'Unsupported file extension: {ext}')


class Error(Exception):
    pass

class InvalidDataForPatchStackError(Error):
    pass


